import QtQuick 1.1
import com.nokia.meego 1.0
import com.marxoft.items 1.0
import "scripts/createobject.js" as ObjectCreator

Page {
    id: root

    function addUrlFromInput(url) {
        UrlChecker.addUrlToQueue(url);
        urlInput.text = "";
    }

    function showTransferDetails(index) {
        var dialog = ObjectCreator.createObject(Qt.resolvedUrl("TransferDetailsDialog.qml"), appWindow.pageStack);
        dialog.transfer = TransferModel.getTransferForQML(index);
        dialog.open();
    }

    function showAboutDialog() {
        var dialog = ObjectCreator.createObject(Qt.resolvedUrl("AboutDialog.qml"), appWindow.pageStack);
        dialog.open();
    }

    function showAddUrlsDialog() {
        var dialog = ObjectCreator.createObject(Qt.resolvedUrl("AddUrlsDialog.qml"), appWindow.pageStack);
        dialog.open();
    }

    function showFileBrowserDialog() {
        var dialog = ObjectCreator.createObject(Qt.resolvedUrl("FileBrowserDialog.qml"), appWindow.pageStack);
        dialog.showFiles = true;
        dialog.startFolder = "/home/user/MyDocs/";
        dialog.open();
        dialog.fileChosen.connect(UrlChecker.importUrlsFromTextFile);
    }

    function showMoveTransfersDialog() {
        var dialog = ObjectCreator.createObject(Qt.resolvedUrl("MoveTransfersDialog.qml"), appWindow.pageStack);
        dialog.open();
    }

    function openMenu() {
        if (TransferModel.count == 0) {
            menu.open();
        }
        else if (TransferModel.rowsSelected) {
            menu3.open();
        }
        else {
            menu2.open();
        }
    }

    orientationLock: Settings.screenOrientation
    tools: ToolBarLayout {

        ToolIcon {
            platformIconId: "toolbar-settings"
            onClicked: appWindow.pageStack.push(Qt.resolvedUrl("SettingsPage.qml"))
        }

        ToolIcon {
            platformIconId: "toolbar-contact"
            onClicked: appWindow.pageStack.push(Qt.resolvedUrl("AccountsPage.qml"))
        }

        ToolIcon {
            platformIconId: "toolbar-view-menu"
            onClicked: root.openMenu()
        }
    }

    Menu {
        id: menu

        MenuLayout {

            MenuItem {
                text: qsTr("Add URLs")
                onClicked: root.showAddUrlsDialog()
            }

            MenuItem {
                text: qsTr("Import URLs")
                onClicked: root.showFileBrowserDialog()
            }

            MenuItem {
                text: qsTr("About")
                onClicked: root.showAboutDialog()
            }
        }
    }

    Menu {
        id: menu2

        MenuLayout {

            MenuItem {
                text: qsTr("Select all")
                onClicked: TransferModel.selectAllRows()
            }

            MenuItem {
                text: qsTr("Add URLs")
                onClicked: root.showAddUrlsDialog()
            }

            MenuItem {
                text: qsTr("Import URLs")
                onClicked: root.showFileBrowserDialog()
            }

            MenuItem {
                text: qsTr("About")
                onClicked: root.showAboutDialog()
            }
        }
    }

    Menu {
        id: menu3

        MenuLayout {

            MenuItem {
                text: qsTr("Select none")
                onClicked: TransferModel.clearSelectedRows()
            }

            MenuItem {
                text: qsTr("Move")
                onClicked: root.showMoveTransfersDialog()
            }

            MenuItem {
                text: qsTr("Pause")
                onClicked: {
                    TransferModel.pauseSelectedTransfers();
                    TransferModel.clearSelectedRows();
                }
            }

            MenuItem {
                text: qsTr("Resume")
                onClicked: {
                    TransferModel.resumeSelectedTransfers();
                    TransferModel.clearSelectedRows();
                }
            }

            MenuItem {
                text: qsTr("Cancel")
                onClicked: {
                    TransferModel.cancelSelectedTransfers();
                    TransferModel.clearSelectedRows();
                }
            }
        }
    }

    TextFieldBox {
        id: urlInput

        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
        }

        z: 1000
        placeholderText: qsTr("Add URL")
        onAccepted: root.addUrlFromInput(text)
        enabled: view.visible
    }

    ListView {
        id: view

        anchors {
            top: urlInput.bottom
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }

        cacheBuffer: 1000
        model: TransferModel
        delegate: TransferDelegate {
            onClicked: TransferModel.rowsSelected ? TransferModel.toggleRowSelected(index) : root.showTransferDetails(index)
            onPressAndHold: TransferModel.toggleRowSelected(index)
        }

        visible: (!urlCheckInfo.visible) && (!progressInfo.visible)
    }

    ScrollDecorator {
        flickableItem: view
    }

    Label {
        anchors.centerIn: parent
        font {
            bold: true
            pixelSize: 40
        }

        color: UI.COLOR_INVERTED_SECONDARY_FOREGROUND
        text: qsTr("No downloads queued")
        visible: (TransferModel.count == 0) && (view.visible)
    }

    UrlCheckInfo {
        id: urlCheckInfo

        anchors {
            top: urlInput.bottom
            topMargin: UI.PADDING_DOUBLE
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            bottomMargin: UI.PADDING_DOUBLE
        }

        visible: false
    }

    ProgressInfo {
        id: progressInfo

        anchors {
            left: parent.left
            leftMargin: 50
            right: parent.right
            rightMargin: 50
            verticalCenter: parent.verticalCenter
        }

        visible: false
    }
}
