#include "categoriesmodel.h"
#include "database.h"

CategoriesModel::CategoriesModel(QObject *parent) :
    QAbstractTableModel(parent)
{
#if QT_VERSION >= 0x040600
    m_roleNames[NameRole] = "name";
    m_roleNames[PathRole] = "path";
#if QT_VERSION < 0x050000
    this->setRoleNames(m_roleNames);
#endif
#endif
    this->loadCategories();
}

CategoriesModel::~CategoriesModel() {}

#if QT_VERSION >= 0x050000
QHash<int, QByteArray> CategoriesModel::roleNames() const {
    return m_roleNames;
}
#endif

int CategoriesModel::rowCount(const QModelIndex &parent) const {
    Q_UNUSED(parent)

    return m_list.size();
}

int CategoriesModel::columnCount(const QModelIndex &parent) const {
    Q_UNUSED(parent)

    return 2;
}

QVariant CategoriesModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if ((role != Qt::DisplayRole) || (orientation != Qt::Horizontal)) {
        return QVariant();
    }

    switch (section) {
    case 0:
        return tr("Name");
    case 1:
        return tr("Download path");
    default:
        return QVariant();
    }
}

QVariant CategoriesModel::data(const QModelIndex &index, int role) const {
    switch (role) {
    case NameRole:
        return m_list.at(index.row()).first;
    case PathRole:
        return m_list.at(index.row()).second;
    case Qt::DisplayRole:
        switch (index.column()) {
        case 0:
            return m_list.at(index.row()).first;
        case 1:
            return m_list.at(index.row()).second;
        default:
            return QVariant();
        }
    default:
        return QVariant();
    }
}

#if QT_VERSION >= 0x040600
QVariant CategoriesModel::data(int row, const QByteArray &role) const {
    return this->data(this->index(row, 0), this->roleNames().key(role));
}
#endif

void CategoriesModel::loadCategories() {
    this->beginRemoveRows(QModelIndex(), 0, this->rowCount() - 1);
    m_list.clear();
    m_list = Database::instance()->getCategories();
    this->endRemoveRows();
    emit countChanged(this->rowCount());
}

void CategoriesModel::addCategory(const QString &name, const QString &path) {
    if (Database::instance()->addCategory(name, path)) {
        this->loadCategories();
    }
}

void CategoriesModel::removeCategory(const QString &name) {
    if (Database::instance()->removeCategory(name)) {
        this->loadCategories();
    }
}

void CategoriesModel::removeCategory(int row) {
    if (Database::instance()->removeCategory(this->data(this->index(row, 0), NameRole).toString())) {
        this->loadCategories();
    }
}
