#ifndef CONNECTION_H
#define CONNECTION_H

#include "enums.h"
#include "file.h"
#include <QNetworkRequest>

class NetworkAccessManager;
class QNetworkReply;

class Connection : public QObject
{
    Q_OBJECT

public:
    explicit Connection(NetworkAccessManager *manager, QObject *parent = 0);
    ~Connection();

    QNetworkRequest request() const;
    void setRequest(const QNetworkRequest &request);

    QByteArray header(const QByteArray &headerName) const;
    void setHeader(const QByteArray &headerName, const QByteArray &value);

    qint64 contentRangeStart() const;
    void setContentRangeStart(qint64 start);

    qint64 contentRangeEnd() const;
    void setContentRangeEnd(qint64 end);

    void setContentRange(qint64 start, qint64 end);

    qint64 position() const;

    QByteArray data() const;
    void setData(const QByteArray &data);

    Transfers::Status status() const;
    QString errorString() const;

public slots:
    void start();
    void pause();
    void cancel();
    void processData();

private:
    void setStatus(Transfers::Status status);
    void setErrorString(const QString &errorString);

private slots:
    void performDownload(const QUrl &url = QUrl());
    void onMetaDataChanged();
    void onReadyRead();
    void onFinished();
    void onDownloadRateLimitChanged(int limit);

signals:
    void dataAvailable(qint64 offset, const QByteArray &data);
    void bytesDownloaded(qint64 bytes);
    void metaInfoReady(MetaInfo info);
    void statusChanged(Transfers::Status status);
    
private:
    NetworkAccessManager *m_nam;
    QNetworkReply *m_reply;
    QNetworkRequest m_request;
    qint64 m_start;
    qint64 m_end;
    qint64 m_downloadedBytes;
    QByteArray m_buffer;
    QByteArray m_data;
    Transfers::Status m_status;
    QString m_errorString;
};

#endif // CONNECTION_H
