#ifndef CROCKO_H
#define CROCKO_H

#include <QObject>
#include <QUrl>
#include "serviceplugin.h"

class QTimer;

class Crocko : public ServicePlugin
{
    Q_OBJECT
    Q_INTERFACES(ServiceInterface)

public:
    explicit Crocko(QObject *parent = 0);
    ServicePlugin* createServicePlugin() { return new Crocko; }
    inline QString iconName() const { return QString("crocko.jpg"); }
    inline QString serviceName() const { return QString("Crocko"); }
    QRegExp urlPattern() const;
    bool urlSupported(const QUrl &url) const;
    void checkUrl(const QUrl &url);
    void getDownloadRequest(const QUrl &url);
    void login(const QString &username, const QString &password);
    inline bool loginSupported() const { return true; }
    inline bool recaptchaRequired() const { return true; }
    inline QString recaptchaKey() const { return m_captchaKey; }
    inline int maximumConnections() const { return m_connections; }
    bool cancelCurrentOperation();

public slots:
    void submitCaptchaResponse(const QString &challenge, const QString &response);

private:
    void startWait(int msecs);

private slots:
    void checkLogin();
    void checkUrlIsValid();
    void onWebPageDownloaded();
    void updateWaitTime();
    void onWaitFinished();
    void getCaptchaUrl();
    void checkCaptchaUrl();

signals:
    void currentOperationCancelled();

private:
    QString m_fileId;
    QString m_captchaId;
    QUrl m_captchaUrl;
    QString m_captchaKey;
    QTimer *m_waitTimer;
    int m_waitTime;
    int m_connections;
};

#endif // CROCKO_H
